Funciones
R
Octubre 2017
1 Funciones internas de R
2 Funciones definidas por el usuario
3 Los argumentos de una funcin
3.1 Forma de pasar los argumentos a una funcin
3.2 Evaluacin perezosa
3.3 El argumento especial "."
3.4 Por qu utilizar funciones
4 Ejercicios
Una funcin es un fragmento de cdigo con un nombre asociado que realiza una serie de tareas y devuelve un valor (incluso si este valor es  NULL). Al definir una funcin se especifica un nombre para este conjunto de sentencias y posteriormente se llamar a esta funcin por ese nombre.

1 Funciones internas de R
R tiene una gran cantidad de funciones internas como mean(), mode(), son(), etc. Adems, al cargar un paquete de R tenemos desde ese momento disponibles todas las funciones que lo componen.

Entre las funciones internas de R encontramos:
  
  Funciones matemticas
Funciones de distribuciones
Funciones cadena
Funciones de cuadro de datos
etc.
2 Funciones definidas por el usuario
Para definir una funcin es necesario especificar el nombre de la funcin y la secuencia de sentencias que se ejecutan cuando esa funcin sea llamada. Una vez definida una funcin, se puede reutilizar una y otra vez a lo largo de todo el programa. Una vez definida una funcin esta puede llamarse como cualquier otra funcin de R.

R no diferencia entre una funcin escrita por un usuario y una funcin del propio R. Esto permite personalizar las funciones de R y tambin desarrollar nuevas funciones rpidamente. Como contrapartida, se realiza un uso intenso y segn algunos autores no muy eficiente de la memoria del ordenador

R dispone de una gran cantidad de funciones tanto en los paquetes que se pueden instalar como funciones escritas por programadores como las creadas por el usuario. Las funciones en R son objetos de primera clase, esto significa que son tratados como cualquier otro objeto de R.

Caractersticas importantes de las funciones en R son:
  
  Las funciones pueden pasarse como argumentos de otra funcin.
Las funciones pueden anidarse (se puede definir una funcin dentro de otra funcin).
El valor que devuelve una funcin es la ltima expresin evaluada dentro del cuerpo de la funcin.
En general se dice que una funcin toma unos argumentos y devuelve un resultado.

El cuerpo de la funcin debe ir encerrado entre llaves en el caso de que haya ms de una sentencia.

{cuerpo_de_la_funcion }
Las funciones se crean utilizando la funcin function() y se almacenan como objetos de R. Las funciones pertenecen a la clase "function".

La sintaxis para definir una funcin es,

nombre <- function (lista_de_parametros) {cuerpo_de_la_funcion}
donde,

Argumento	Descripcin
nombre	Es el nombre de la funcin que se define.
function	Es la palabra clave con la que se declara la funcin.
lista de parmetros	Son los valores, variables u otros objetos de R que se pasan como entrada de la funcin.
cuerpo de la funcin	Son las sentencias que constituyen la funcin.
3 Los argumentos de una funcin
Propiedades

Los argumentos de las funciones son argumentos con nombre.
Las funciones en R pueden tener argumentos con valores por defecto, con valor nulo (NULL) o con valor missing.
Los argumentos formales de una funcin son los que se declaran en la definicin de la funcin.
La funcin formals() devuelve la lista de los argumentos de una funcin.
La funcin body() devuelve el cuerpo de la funcin.
No es obligatorio incluir todos los argumentos en la llamada a una funcin.
3.1 Forma de pasar los argumentos a una funcin
Los argumentos de una funcin pueden pasarse por:
  
  nombre
por posicin
una mezcla de por nombre y posicin
Ejemplo 1

mydata <- rnorm(100)               # Genera 100 valores provenientes de una N(0,1)
sd(mydata)                         # Pasa el parmetro por posicin
## [1] 1.158897
sd(x = mydata)                     # Pasa el parmetro por nombre
## [1] 1.158897
sd(x = mydata, na.rm = FALSE)      # Pasa el parmetro por nombre
## [1] 1.158897
sd(na.rm = FALSE, x = mydata)      # Pasa el parmetro por nombre
## [1] 1.158897
sd(na.rm = FALSE, mydata)          # Pasa el parmetro por nombre y el segundo por posicin
## [1] 1.158897
Cuando un argumento es pasado por nombre, sale del orden de la lista de los argumentos de la funcin, mientras que el resto de los argumentos mantienen el orden en el que fueron declarados en la funcin.

args(lm)
## function (formula, data, subset, weights, na.action, method = "qr", 
##     model = TRUE, x = FALSE, y = FALSE, qr = TRUE, singular.ok = TRUE, 
##     contrasts = NULL, offset, ...) 
## NULL
Los siguientes cdigos son equivalentes:
  
  lm(data = mydata, y ~ x, model = FALSE, 1:100)
lm(y ~ x, mydata, 1:100, model = FALSE)
En general es recomendable pasar los argumentos por nombre para evitar posibles confusiones.

En R el proceso de emparejar valores con argumentos sigue estos pasos:
  
  Empareja los argumentos que coinciden totalmente con el nombre.
Empareja los argumentos que coinciden parcialmente con el nombre.
Empareja los argumentos de forma posicional.
3.2 Evaluacin perezosa
Los argumentos de una funcin son evaluados nicamente si es necesario. Esta propiedad se conoce como "evaluacin perezosa" (lazy evaluation),

Ejemplo 2 En la siguiente funcin f() hay dos arguemntos el a y el b. Cuando se ejecuta f(2) el argumento a=2 y el b permanece missing. Como b no interviene en el clculo del valor de la funcin no se produce error, es decir R nunca se pregunta por el valor de b y por lo tanto no se entera que no ha sido especificado.

f <- function(a, b) {
  a^2
}
f(2)
## [1] 4
En este segundo ejemplo is falta b, R produce un mensaje de error.

f <- function(a, b) {
  print(a)
  print(b)
}
f(45)
## [1] 45
## Error in print(b): el argumento "b" est ausente, sin valor por omisin
Como no se ha pasado el valor de "b", R produce un mensaje de error al intentar evaluar print(b).

Se puede ver como la ejecucin es secuencial: primero evala print(a) y despus print(b).

3.3 El argumento especial "."
El argumento . indica que hay un nmero no determinado de argumentos que son pasados a travs de otra funcin de forma general. Se suele utilizar . cuando se invoca a otra funcin y no se quiere copiar toda la lista de argumentos de la funcin original.

miplot <- function(x, y, type = "l", ...) {
  plot(x, y, type = type, ...)
}
Las funciones genricas utilizan . para pasar argumentos en funcin del mtodo al que se aplique.

Una funcin es genrica cuando se "adapta" al objeto al que se aplica.

mean
## function (x, ...) 
## UseMethod("mean")
## <bytecode: 0x0000000011fd39b8>
## <environment: namespace:base>
El argumento . es necesario cuando no se sabe el nmero de argumentos que se va a pasar a una funcin.

args(paste)
## function (..., sep = " ", collapse = NULL) 
## NULL
args(cat)
## function (..., file = "", sep = " ", fill = FALSE, labels = NULL, 
##     append = FALSE) 
## NULL
Una vez que se utiliza . el resto de los argumentos deben pasarse por nombre y no se permite el emparejado parcial.

args(paste)
## function (..., sep = " ", collapse = NULL) 
## NULL
paste("a", "b", sep = ":")
## [1] "a:b"
paste("a", "b", se = ":")
## [1] "a b :"
Ejemplo 3: Funcin para calcular la mediana.

En R se dispone de la funcin median() para calcular la mediana de un conjunto de datos. Sin embargo, si queremos tener nuestra propia funcin podemos escribir la siguiente.

Nuestra funcin ordena el conjunto de datos y toma el elemento que est en el medio.

nuestramediana <- function(x){
  n <- length(x)
  return(sort(x)[(n+1)/2])
}
La comprobamos con un ejemplo,

nuestramediana <- function(x){
  n<-length(x)
  return(sort(x)[(n+1)/2])
}

x <- c(1,2,3,4,5)
mean(x)
## [1] 3
nuestramediana(x)
## [1] 3
La nueva funcin media se llama nuestramedia(). Tiene un nico parmetro que es el conjunto de datos a analizar. El cuerpo de la funcin esta encerrado entre llaves para poder declarar varias sentencias. El valor que devuelve est definido en la funcin return().

La funcin return() no es siempre necesaria dentro de una funcin. Por defecto R devuelve el ltimo valor calculado.

Ejemplo 4: Solucin polinomio de segundo grado

# Polinmio de grado 2
poli_dos <- function(a, b, c,x) { a*x^2 + b*x + c}
y <- poli_dos(2,3,4,10)
print (y)
## [1] 234
Los parmetros se pasan a la funcin por posicin y no por valor. Esto permite utilizar los mismos nombres dentro y fuera de la funcin, ya que para R son objetos distintos.

Ejemplo 5: Funcin logaritmo en base 5

# Clculo del logaritmo en base 5 de un nmero

# devuelve el logaritmo en base 5 de un nmero
log5 <- function(x){
  y <- log(x) / log(5)
  return(y)
}
Ejemplos de funciones de R son:
  
  Grficos de alto nivel

y <- c(234, 345, 234, 634, 567, 234, 45,125, 829)
hist(y)


# Para saber ms sobre hist() escribir ?hist()
#?hist
Grficos de bajo nivel

La funcin plot() es la funcin grfica ms importante de las de bajo nivel en R.

y <- c(234, 345, 234, 634, 567, 234, 45,125, 829)
plot(y)
grid()


# Para saber ms sobre plot() y sobre grid()
#?plot
#?grid()
Otra funcin grfica til es curve().

curve(sin, -pi,pi)


3.4 Por qu utilizar funciones
Las ventajas de dividir un programa en funciones son mltiples entre ellas sealaremos:
  
  Al agrupar un conjunto de sentencias bajo un nombre hace que el programa sea ms fcil de leer, entender, actualizar y depurar
Las funciones permiten no repetir cdigo innecesariamente por lo que los programas son ms cortos.
Una funcin puede utilizarse en muchos programas. Es decir, permite la reutilizacin de un cdigo en diferentes programas.



4 Ejercicios

1.- Sea la funcin y=f(x) definida por

f(x)=??????????????????x3x2x?????????x???00<x???1x>1

Se pide que escriba una funcin en R de forma que para cada x calcule su correspondiente y. Una vez escrita su funcin utilice el cdigo siguiente para dibujar la funcin.

# Introduccin
valores.x <- seq(-2,2, by=0.1)

# Para cada x se calcula su y

n <- length(valores.x)
valores.y <- rep(0,n)
for (i in 1:n) {
  x <- valores.x[i]
  # Aqu tiene que ir su funcin que calcula los distintos valores de y
  valores.y[i] <- funcin_propia(x[i])
}
# Salida
plot(valores.x, valores.y, type = "l")
el dibujo debe ser parecido al siguiente grfico,

A la vista del grfico anterior, La funcin f tiene derivada en 0? y en 1? Se podra vectorizar el programa utilizando la funcin ifelse().

2)Sea h(x,n)=1+x+x2+???+xn=???ni=0xi . Escribe una funcin en R para calcular h(x,n) utilizando un bucle for.

3)La funcin del ejercicio 2, h(x,n), es la suma finita de una serie geomtrica. Se sabe que la formula explcita para x???1 es, h(x,n)=(1???xn+1)/(1???x).

Comprueba tu programa sabiendo que,

x	n	h(x,n)
0.3	55	1.428571
6.6	8	4243335.538178

4)Escribe un programa que obtenga el mismo resultado que el ejercicio 2 pero utilizando while en vez de for. Despus escribe una funcin que utiliza operaciones vectoriales en vez de bucles.

5)Para rotar un vector (x,y)T en sentido anti horario ?? radianes hay que multiplicar al vector por la derecha por la matriz,
(cos(??)sin(??)sin(?????)cos(??))

Escribe una funcin en R que lo calcule.

6)Dado un vector x, calcular su media geomtrica utilizando un bucle y operaciones vectoriales. (La media geomtrica de x1,x2,???,xn es (???ni=1xi)1/n. Puedes comprobar que la media armnica (???ni=11/xi)???1 cuando los a xi son positivos, es siempre menor o igual que la media geomtrica, esta a su vez es siempre menor o igual que la media aritmtica.
                                                                                               
7)Escribe una funcin que calcule el mnimo de un vector sin utilizar ninguna funcin de R, en particular las funciones sort() y min().
                                                                                               
8)El juego de los dados es como sigue. Primero se lanzan dos dados, sea x la suma de los dados. Si x=7 o x=11 ganas, en otro caso sigues tirando hasta obtener la tirada de la primera vez, en cuyo caso ganas o hasta obtener 7 u 11 en cuyo caso pierdes. Escribe un programa que simule el juego. Puedes utilizar la siguiente lnea que simula el lanzamiento de dos dados,  x <-  sum(ceiling(6*runif(2))).
                                                                                               
9)Supongamos que (x(t),y(t)) tienen como coordenadas polares (t???,2??t). Dibuja un grfico (x(t),y(t)) para t???[0,10]. Debera tener el aspecto siguiente,
                                                                                               
            

#EN EXAMEN ENTRA FUNCIN A TROZOS                                                                                   