Las Tablas de Datos
R
1 Introduccin
2 Crear un data frame
3 Acceder a los elementos de una tabla de datos.
3.1 Como matriz bidimensional
3.2 Como lista
3.3 Las funciones attach() y detach()
4 Seleccionar casos, variables y transformar variables
4.1 La funcin subset()
4.2 La funcin transform()
4.3 Aadir una nueva variable.
4.4 Las funciones with() y within()
4.5 Eliminar una variable (columna)
5 Ordenar un data frame
6 Combinacin de los data frame
6.1 La funcin merge(): Combinar dos data frame
6.1.1 Dos data frame con variable clave comn y con el mismo nombre
6.1.2 Dos data frame con variable clave comn y con nombre distinto
6.1.3 Dos data frame con variable clave comn y variables repetidas
6.1.4 Dos data frame sin una variable clave comn como campo clave
6.2 Combinar ms de dos data frame. La funcin Reduce()
7 Eliminar filas duplicadas en un data frame
8 Dividir un data frame. La funcin split()
9 Familia apply en data frame
9.1 La funcin lapply()
9.2 La funcin by()
9.3 La funcin tapply()
10 Lectura de datos
11 Atributos de un data frame
1 Introduccin
Un data frame es un objeto de R que permite almacenar datos bidimensionales. Tienen en comn con las matrices que son objetos rectangulares donde todas las columnas deben tener la misma longitud, pero a diferencias de ellas cada columna puede ser de tipo numrico, carcter o lgico. De igual forma, pueden verse como una lista donde todas sus elementos deben tener la misma longitud.

Se puede ver como un archivo de datos rectangular donde las filas son los casos y las columnas son las variables que se mide en cada caso. Un data frame es equivalente a un fichero de datos de SPSS o de SAS. Sin embargo, un data frame no es un fichero sino un objeto.

Un data frame como un dataset de SPSS o de SAS es un fichero rectangular con forma de matriz donde la primera columna puede contener los nombres de los individuos y la primera fila los nombres de las variables.

Las columnas (variables) no numricas, en un data frame, son consideradas como factores, con tantos niveles como valores distintos haya (en los datos, salvo que se indique otra cantidad).

2 Crear un data frame
Un data frame puede crearse de diferentes formas, entre ellas se destacan:
  
  Por medio de la funcin data.frame().
Leyendo un archivo de datos con read.table().
son las dos formas ms comunes de obtener un data frame.

En el ejemplo siguiente se crea una tabla de datos a partir de vectores y factores previamente definidos.

Ejemplo 1

# Se crea el vector edad por medio de la funcin c().
EDAD<-c(12,15,17,10,45)

# Se crea el vector CASO por medio de la funcin seq() 
CASO<- seq(1,5)

# Se crea el factor SEXO por medio de la funcin rep().
SEXO<-rep(x=c("M","H"),times=c(2,3))

# Creamos el data frame por medio de la funcin data.frame().

df<-data.frame(edad=EDAD,caso=CASO,sexo=SEXO)
df
##   edad caso sexo
## 1   12    1    M
## 2   15    2    M
## 3   17    3    H
## 4   10    4    H
## 5   45    5    H
Ejemplo 2

resultados <- data.frame(
  talla = c(185,172,165,177,168),
  peso  = c( 82, 70, 76, 52, 72),
  QI    = c(110,120, 92,120,142),
  sexo  = c("H","M","M", "M","H"),
  row.names= c("Pablo", "Marta","Lola","Genoveva", "Luis"))
resultados
##          talla peso  QI sexo
## Pablo      185   82 110    H
## Marta      172   70 120    M
## Lola       165   76  92    M
## Genoveva   177   52 120    M
## Luis       168   72 142    H
3 Acceder a los elementos de una tabla de datos.
Los data frame comparten caractersticas con las matrices y con las listas. Para extraer elementos de las tablas de datos se pueden referenciar sus elementos como si fuese una matriz y como una lista.

3.1 Como matriz bidimensional
Los elementos se pueden extraer utilizando las referencias de una matriz bidimensional.

resultados[2,3]
## [1] 120
resultados[4,]
##          talla peso  QI sexo
## Genoveva   177   52 120    M
resultados[4, c(2,4)]
##          peso sexo
## Genoveva   52    M
3.2 Como lista
Para trabajar con las variables (columnas) de un data.frame se puede utilizar la notacin estndar de las listas,

nombre_data_frame$nombre_columna

nombre_data_frame[[ ]]

Las dos formas son equivalentes en el sentido que muestran la misma informacin. Sin embargo, no se obtiene el mismo objeto.

Ejemplo 3

df[3]
##   sexo
## 1    M
## 2    M
## 3    H
## 4    H
## 5    H
dim(df[3])
## [1] 5 1
dim(df$sexo)
## NULL
A veces, puede ser ms interesante emplear simplemente el nombre de la columna.

3.3 Las funciones attach() y detach()
La accin de vincular los nombres de las columnas con sus nombres como vectores se realiza por medio de la funcin attach(). En este ejemplo, para mostrar los nombres del data frame se ejecuta la sentencia attach(fd).

Ejemplo 4

attach(df)
names(df)
## [1] "edad" "caso" "sexo"
edad+caso
## [1] 13 17 20 14 50
(edad - mean(edad))/sd(edad)
## [1] -0.5438479 -0.3346756 -0.1952275 -0.6832961  1.7570471
detach(df)
IMPORTANTE

Una vez vinculado un data frame por medio de la funcin attach() las modificaciones hechas a las columnas por medio de las referencia de sus nombres no afectan al data frame original. A efectos prcticos, al ejecutar attach(df), R realiza una copia del data frame 'df' y es en esta copia donde se trabaja.

Por ejemplo, si se ejecuta

attach(df)
edad <- 2*edad
edad
## [1] 24 30 34 20 90
detach(df)
df$edad
## [1] 12 15 17 10 45
Se puede apreciar que el data frame 'df' no ha sido modificado.

Para deshacer la vinculacin se ejecuta la funcin detach(). En general se debe evitar el uso de la funcin attach().

4 Seleccionar casos, variables y transformar variables
Hay dos formas de seleccionar casos y variables en un data frame. La primera es utilizar las tcnicas usuales de matrices, la seleccin por ndices. La segunda es utilizar las funciones; subset() y transform(). Aunque el uso de las funciones resulta ms cmodo la seleccin por medio de ndices es ms poderosa.

4.1 La funcin subset()
Permite seleccionar casos (filas). Su sintaxis es:
  
  subset( data.frame, condicin lgica, select=vector con nombres de variables)
Ejemplo 5

df.2 <- subset(df,edad>14,select=c(edad,sexo))
df.2
##   edad sexo
## 2   15    M
## 3   17    H
## 5   45    H
4.2 La funcin transform()
Para transformar unas determinadas columnas, o crear nuevas variables, se puede utilizar la funcin transform(). Su sintaxis es,

transform(data.frame,variable1=f(varibles),variable3=f(varibles)...)
Ejemplo 6

En este ejemplo se transforma la variable edad y el resultado se almacena en un nuevo data.frame llamado, 'fd.3'.

df.3 <- transform(df,edad=edad*2)
df.3
##   edad caso sexo
## 1   24    1    M
## 2   30    2    M
## 3   34    3    H
## 4   20    4    H
## 5   90    5    H
Para realizar la misma operacin sin la funcin transform() bastara con escribir

df$edad <-  df$edad*2

4.3 Aadir una nueva variable.
Con transform().

df.3 <- transform(df, edad2 = edad+23)
df.3
##   edad caso sexo edad2
## 1   12    1    M    35
## 2   15    2    M    38
## 3   17    3    H    40
## 4   10    4    H    33
## 5   45    5    H    68
Sin transform().

Construimos la variable: edad.n <- df$edad + 23
Aadimos la variable por medio del operador de asignacin, "<-"
df$edad.nueva <- edad.n
df
##   edad caso sexo
## 1   12    1    M
## 2   15    2    M
## 3   17    3    H
## 4   10    4    H
## 5   45    5    H
4.4 Las funciones with() y within()
Otras opciones para transformar variables son utilizar las funciones with() y within() que permiten realizar transformaciones en un data frame. Veamos los siguientes ejemplos.

Ejemplo 7

N <- 3
df <- data.frame(x1=rnorm(N),x2=runif(N))
df$y <- with(df,{x1+x2})
df
##           x1        x2         y
## 1 -0.6718092 0.9838237 0.3120145
## 2  0.2531490 0.0265251 0.2796741
## 3  1.1335580 0.1975151 1.3310731
df <- within(df,{
  x1.sq <- x1^2
  x2.sq <- x2^2
  y <- x1.sq+x2.sq
  x1 <- x2 <- NULL
})
df
##            y        x2.sq      x1.sq
## 1 1.41923678 0.9679091288 0.45132765
## 2 0.06478801 0.0007035811 0.06408443
## 3 1.32396594 0.0390122324 1.28495371
4.5 Eliminar una variable (columna)
Una variable de un data frame se puede eliminar directamente referenciandola por el nmero de columna con signo negativo, df [, -n] elimina la columna n

Ejemplo 8

misdatos <- data.frame(x=1:10, y=rnorm(10))
misdatos
##     x          y
## 1   1  0.1785254
## 2   2 -0.5778917
## 3   3  0.3432434
## 4   4  2.1391367
## 5   5  0.4856955
## 6   6 -0.7563755
## 7   7  1.5379155
## 8   8  0.7385278
## 9   9  0.3862108
## 10 10 -1.2564146
Para eliminar la variable "x" :
  
  misdatos[, -1]
##  [1]  0.1785254 -0.5778917  0.3432434  2.1391367  0.4856955 -0.7563755
##  [7]  1.5379155  0.7385278  0.3862108 -1.2564146
Por medio de la funcin NULL.

misdatos <- data.frame(x=1:10, y=rnorm(10))
misdatos
##     x           y
## 1   1 -0.04487362
## 2   2  0.56633148
## 3   3  1.39895496
## 4   4  1.93096941
## 5   5 -0.61063796
## 6   6  1.88727615
## 7   7  0.07790746
## 8   8  0.40230824
## 9   9 -0.08099190
## 10 10 -0.22435952
misdatos[, -1]
##  [1] -0.04487362  0.56633148  1.39895496  1.93096941 -0.61063796
##  [6]  1.88727615  0.07790746  0.40230824 -0.08099190 -0.22435952
misdatos
##     x           y
## 1   1 -0.04487362
## 2   2  0.56633148
## 3   3  1.39895496
## 4   4  1.93096941
## 5   5 -0.61063796
## 6   6  1.88727615
## 7   7  0.07790746
## 8   8  0.40230824
## 9   9 -0.08099190
## 10 10 -0.22435952
misdatos[,1] <- NULL
misdatos
##              y
## 1  -0.04487362
## 2   0.56633148
## 3   1.39895496
## 4   1.93096941
## 5  -0.61063796
## 6   1.88727615
## 7   0.07790746
## 8   0.40230824
## 9  -0.08099190
## 10 -0.22435952
Los efectos de los dos mtodos no son exactamente iguales. Estudia las diferencias.

5 Ordenar un data frame
Para ordenar un data frame se disponen de diferentes opciones. La principal es ordenar los ndices de las filas en funcin de una o ms columnas por medio de la funcin order().

La funcin order() devuelve un vector de ndices permutados, de tal forma que sus argumentos quedan ordenados de forma ascendente. Si hay ms de un vector,en caso de empate en el primer vector se utilizan los valores del segundo. Si persiste el empate se mirara el tercero, y as sucesivamente.

sort.list tiene la misma funcin pero utilizando nicamente un argumento.

Sintaxis

order(..., na.last = TRUE, decreasing = FALSE)
sort.list(x, partial = NULL, na.last = TRUE, decreasing = FALSE,
          method = c("shell", "quick", "radix"))
Argumentos	
.	Una secuencia de vectores numricos, complejos, de caracter o lgicos todos de igual longitud o un objeto de clase R.x Un vector.
partial	Vector de ndices para una ordenacin parcial. (Los valores nulos no se utilizan)
decreasing	Logica. Determina si el orden es creciente o decreciente.
na.last	Controla el tratamiento de los NAs. Si es TRUE, los valores missing se colocan al final de los datos, si es FALSE, se colocan al principio de los datos, si NA, se eliminan de los da-tos.
method	El mtodo que se empleara. Los emparejamientos parciales est?n permitidos.
Detalles

En el caso de empates en el primer vector, se utilizarn los valores del segundo vector para deshacer los empates. Si permanece el empate, los valores de los siguientes argumentos se utilizaran para romper el empate. El tipo de ordenacin (sort) empleado es "stable" salvo para el mtodo "quick" ( method = "quick"), si quedar algun empate sin resolver se dejaran en su orden original.

Los nmeros complejos se ordenan primero segn su parte real y luego por la parte imaginaria. El orden de los vectores carcter depender de la secuencia local: ver comparison.

El mtodo por defecto para sort.list es un compromiso. El mtodo "quick" se utiliza nicamente para valores numricos con la opcin na.last=NA, y no es estable, sin embargo es el ms rpido en vectores grandes. El metodo "radix" esta nicamente implementado para valores enteros menores de 100.000. En esta situacin, es (muy) y estable. Es el mtodo de eleccin para ordenar factores.

El mtodo partial se mantiene para permitir la compatibilidad de otras implementaciones de S, pero no permite otros valores y la ordenacin ser siempre completa.

Ejemplos 9

# La funcin order() da como resultado los ndices de ordenacin.
# La funcin sort() da como resultado el objeto ordenado

# Vector caracter

c1 <- c("A","B","C","D")
sort(c1,decreasing = TRUE)
## [1] "D" "C" "B" "A"
order(c1)
## [1] 1 2 3 4
c2 <- c("A","D","C","B")
sort(c2,decreasing = TRUE)
## [1] "D" "C" "B" "A"
order(c2)
## [1] 1 4 3 2
# Vector numrico

x <- c(2,1:9)
x_sort <- sort(x)

x
##  [1] 2 1 2 3 4 5 6 7 8 9
(indice <- order(x))
##  [1]  2  1  3  4  5  6  7  8  9 10
x[indice]
##  [1] 1 2 2 3 4 5 6 7 8 9
x_sort
##  [1] 1 2 2 3 4 5 6 7 8 9
# Ordenar un data.frame

df1 <- data.frame(
  pais = c("M","G","J","L","Z","B"),
  pob = c(234, 456, 267,987,123,369),
  area = c(1,2,3,2,3,1))

df2 <- df1[order(df1$pais),]
df2
##   pais pob area
## 6    B 369    1
## 2    G 456    2
## 3    J 267    3
## 4    L 987    2
## 1    M 234    1
## 5    Z 123    3
df3 <- df1[order(df1$pob),]
df3
##   pais pob area
## 5    Z 123    3
## 1    M 234    1
## 3    J 267    3
## 6    B 369    1
## 2    G 456    2
## 4    L 987    2
df4 <- df1[order(df1$area),]
df4
##   pais pob area
## 1    M 234    1
## 6    B 369    1
## 2    G 456    2
## 4    L 987    2
## 3    J 267    3
## 5    Z 123    3
# funcin xtfrm() es una funcin auxiliar que produce un vector numrico que puede ser ordenado en el mismo orden que 'x'.

df5 <- df1[order(-df1$area, -xtfrm(df1$pais)),]
df5
##   pais pob area
## 5    Z 123    3
## 3    J 267    3
## 4    L 987    2
## 2    G 456    2
## 1    M 234    1
## 6    B 369    1
# Quiero ordenar un dataframe por varias columnas en R
# Por ejemplo, con los datos siguientes se quiere ordenar en primer lugar por
#  la columna z (descendente) y en segundo lugar por la columna b (ascendente):

dd <- data.frame(b = factor(c("Hi", "Med", "Hi", "Low"),
                            levels = c("Low", "Med", "Hi"), ordered = TRUE),
                 x = c("A", "D", "A", "C"), y = c(8, 3, 9, 9),
                 z = c(1, 1, 1, 2))
dd
##     b x y z
## 1  Hi A 8 1
## 2 Med D 3 1
## 3  Hi A 9 1
## 4 Low C 9 2
dd[with(dd, order(b,-z)), ]
##     b x y z
## 4 Low C 9 2
## 2 Med D 3 1
## 1  Hi A 8 1
## 3  Hi A 9 1
6 Combinacin de los data frame
Dentro de los trabajos con data frame una de las tareas ms comunes es la fusin de dos data frame. Las formas de fusionarlos son:
  
  Agregar filas: Dados los data frame X e Y agregar filas consiste en "pegar" las filas de un conjunto de datos debajo del otro. Es lo que se conoce como unin vertical y se realiza con la funcin rbind(X,Y).

Agregar columnas: Dados los data frame X e Y agregar columnas consiste en "pegar" las columnas de un conjunto de datos al lado del otro. Es lo que se conoce como unin horizontal y se realiza con la funcin cbind(X,Y).

Join o vlookup: Dados los data frame X e Y al primer data frame se le agrega el contenido del segundo. Generalmente, los data frame presentan una o varias variables que funcionan como campos claves o "keys". Suponiendo que cada data frame contiene al menos una key comun a ambos se puede clasificar los join en:
  
  Inner join. Devuelve nicamente los casos en los que la key se repite en ambos data frame.
Left join. Devuelve todos los casos del data frame de la izquierda y slo los casos del conjunto de la derecha que tiene una key que coincida con algn elemento de la izquierda.
Right join. Devuelve todos los casos del data frame de la derecha y slo los casos del conjunto de la izquierda que tiene una key que coincida con algn elemento de la derecha.
Full join. Devuelde todas las observaciones tanto del data frame de la izquierda como de la derecha.
todos estos tipos de combinacin de data frame se realizan con la funcin merge().

6.1 La funcin merge(): Combinar dos data frame
La funcin merge() combina dos data frame columna a columna o por los nombres de filas.

Sintaxis

merge(x, y, ..)

merge(x, y, by = intersect(names(x), names(y)),
      
      by.x = by, by.y = by, all = FALSE, all.x = all, all.y = all,
      
      sort = TRUE, suffixes = c(".x",".y"), incomparables = NULL, ...)
Argumentos	
x, y	data frames, u objetos que se van a combinar en un objeto
by, by.x, by.y	especificaci?n de las columnas comunes.
all	logica; all = L es una abreviatura de all.x = L y all.y = L.
all.x	logica; si TRUE, se aade una fila extra a la salida, una fila por cada fila en x que no tiene correspondencia con una fila en y. Estas filas tendrn NAs en aquellas columnas que sean rellenado con campos de y. El valor por defecto es FALSE, en este caso nicamente las filas con datos en los dos data frame se incluirn en la salida.
all.y	logica; analoga a all.x -
  sort	logica. El resultado se ordenar en funcin de las columnas especificadas en by .
suffixes	caracter(2) especifica los sufijos que se utilizaran para crear nombres cuando los nombres especificados en by names() no sean nicos.
incomparables	Valores que no pueden ser emparejados. Ver match.
.	Argumentos que son pasados a o de mtodos.
Detalles

Por defecto los data frames se combinan en funcin de las columnas que en los dos data frame tienen nombres comunes, pero se puede especificar las columnas que se quiere utilizar por medio de by.x y by.y, lo que permite realizar la unin sin que en los dos data frame se utilizan los mismos nombres de columna.

Las columnas pueden especificarse por nombre, nmero o por medio de un vector lgico, llamado row.names .El nmero 0 especifica la fila de nombres. Las filas en los dos data frame que van a emparejarse son extra?das y juntadas. Si hay ms de un posible emparejamiento, se realizan tantos emparejamientos como sea posible y cada uno de ellos genera una nueva fila. El significado preciso de 'match', se puede ver en match.

Si by o by.x y by.y tiene longitud 0 (un vector de longitud cero o NULL), el resultado, r, es el producto cartesiano de x e y, es decir,  dim(r) = c(nrow(x)*nrow(y), ncol(x) + ncol(y)).

Si all.x es TRUE, todos los casos no emparejados de x se aaden al resultado, aadiendo NA en las columnas correspondientes de y; de igual forma para all.y.

Si le resto de columnas en los data frames tuviesen nombres iguales se les aadira como sufijo (".x" and ".y" por defecto) para conseguir que no haya repeticin de nombres.

Valor

Un data frame. Las filas se ordenan por defecto segn el orden lexicogrfico entre las columnas comunes, pero si la opcin sort = FALSE se ordenaran en un orden no especificado. Primero las columnas que son comunes, seguidas de las columnas de x e y. Si en el emparejamiento se han especificado los nombres de las filas, se aade una columna extra (carcter) a la izquierda que se llama row.names.

6.1.1 Dos data frame con variable clave comn y con el mismo nombre
Ejemplo 10 Tabla de datos con una columna en comn y con el mismo nombre.

# Data frame x
x <- data.frame(id=c(1,2,3,4,5),
                nombre=c("Jos","Pilar","Nuria","Pepa","Antonio"),
                edad=c(30,40,22,53,27),
                peso=c(75,60,55,78,80))
x
##   id  nombre edad peso
## 1  1    Jos   30   75
## 2  2   Pilar   40   60
## 3  3   Nuria   22   55
## 4  4    Pepa   53   78
## 5  5 Antonio   27   80
#Data frame y
y <- data.frame(id=c(1,2,3,4,5),
                carnet=c("si","si","si","no","no"),
                estad.civil=c("soltero","casado","soltero","divorciado","soltero"),
                hijos=c("no","si","no","si","no"),
                n.hijos=c(NA,2,NA,1,NA))
y
##   id carnet estad.civil hijos n.hijos
## 1  1     si     soltero    no      NA
## 2  2     si      casado    si       2
## 3  3     si     soltero    no      NA
## 4  4     no  divorciado    si       1
## 5  5     no     soltero    no      NA
Los dos data frame tienen una columna en comn, en el sentido que existe una columna que tienen los mismos elementos en los dos objetos y adems tienen el mismo nombre. Para unirlas utilizamos el siguiente cdigo:
  
  merge(x,y,by="id")
##   id  nombre edad peso carnet estad.civil hijos n.hijos
## 1  1    Jos   30   75     si     soltero    no      NA
## 2  2   Pilar   40   60     si      casado    si       2
## 3  3   Nuria   22   55     si     soltero    no      NA
## 4  4    Pepa   53   78     no  divorciado    si       1
## 5  5 Antonio   27   80     no     soltero    no      NA
6.1.2 Dos data frame con variable clave comn y con nombre distinto
Ejemplos 11 Dos data frame con variable clave comn y con nombre distinto

Vamos a trabajar con los data frame authors y books.

## use character columns of names to get sensible sort order
authors <- data.frame(
  surname = I(c("Tukey", "Venables", "Tierney", "Ripley", "McNeil")),
  nationality = c("US", "Australia", "US", "UK", "Australia"),
  deceased = c("yes", rep("no", 4)))
El data frame 'authors' consta de 4 columnas (variables):
  
  surname (apellido)
nationality (nacionalidad)
deceased (fallecimiento)
authors
##    surname nationality deceased
## 1    Tukey          US      yes
## 2 Venables   Australia       no
## 3  Tierney          US       no
## 4   Ripley          UK       no
## 5   McNeil   Australia       no
books <- data.frame(
  name = I(c("Tukey", "Venables", "Tierney",
             "Ripley", "Ripley", "McNeil", "R Core")),
  title = c("Exploratory Data Analysis",
            "Modern Applied Statistics ...",
            "LISP-STAT",
            "Spatial Statistics", "Stochastic Simulation",
            "Interactive Data Analysis",
            "An Introduction to R"),
  other.author = c(NA, "Ripley", NA, NA, NA, NA,
                   "Venables & Smith"))
El data frame 'books' consta de 3 columnas (variables):
  
  name (nombre),
title (ttulo)
other.authors (otros autores)
books
##       name                         title     other.author
## 1    Tukey     Exploratory Data Analysis             <NA>
## 2 Venables Modern Applied Statistics ...           Ripley
## 3  Tierney                     LISP-STAT             <NA>
## 4   Ripley            Spatial Statistics             <NA>
## 5   Ripley         Stochastic Simulation             <NA>
## 6   McNeil     Interactive Data Analysis             <NA>
## 7   R Core          An Introduction to R Venables & Smith
Lo data frame 'authors' y 'books' tiene en comn la informacin del apellido del autor de la obra. Sin embargo, en 'authors' esta variable se llama 'surname' y en 'book' 'name'.

Para unir estos dos data frame utilizaremos como campo clave estas variables.

Por medio de

by.x = ??? se especifica el nombre de la variable en el primer data frame (authors)
by.y = ??? se especifica el nombre de la variable en el segundo data frame (books)
Primero vamos a unir, authors con books:
  
  (m1 <- merge(authors, books, by.x = "surname", by.y = "name"))
##    surname nationality deceased                         title other.author
## 1   McNeil   Australia       no     Interactive Data Analysis         <NA>
## 2   Ripley          UK       no            Spatial Statistics         <NA>
## 3   Ripley          UK       no         Stochastic Simulation         <NA>
## 4  Tierney          US       no                     LISP-STAT         <NA>
## 5    Tukey          US      yes     Exploratory Data Analysis         <NA>
## 6 Venables   Australia       no Modern Applied Statistics ...       Ripley
En este caso, primero aparecen las columnas (variables) del data frame 'authors' seguidos de las columnas del data frame 'books'. El campo comun, declarado en by.x= y en by.y=es el que indexa el nuevo data frame y mantiene el nombre de del primer data frame. En este caso el nombre que aparece en el data frame 'authors' que es name.

(m2 <- merge(books, authors, by.x = "name", by.y = "surname"))
##       name                         title other.author nationality deceased
## 1   McNeil     Interactive Data Analysis         <NA>   Australia       no
## 2   Ripley            Spatial Statistics         <NA>          UK       no
## 3   Ripley         Stochastic Simulation         <NA>          UK       no
## 4  Tierney                     LISP-STAT         <NA>          US       no
## 5    Tukey     Exploratory Data Analysis         <NA>          US      yes
## 6 Venables Modern Applied Statistics ...       Ripley   Australia       no
Si ahora unimos books con authors, primero aparecen las columnas (variables) del data frame 'books' seguidos de los del data frame 'authors' indexados por la variable surname que es el nombre de la columna comn en el primer data frame de la unin.

6.1.3 Dos data frame con variable clave comn y variables repetidas
Volviendo al ejemplo anterio supongamos que en el data frame y aparece la columna peso pero con valores distintos.

# Data frame x
x <- data.frame(id=c(1,2,3,4,5),
                nombre=c("Jos","Pilar","Nuria","Pepa","Antonio"),
                edad=c(30,40,22,53,27),
                peso=c(75,60,55,78,80))
x
##   id  nombre edad peso
## 1  1    Jos   30   75
## 2  2   Pilar   40   60
## 3  3   Nuria   22   55
## 4  4    Pepa   53   78
## 5  5 Antonio   27   80
#Data frame y
y <- data.frame(id=c(1,2,3,4,5),
                carnet=c("si","si","si","no","no"),
                estad.civil=c("soltero","casado","soltero","divorciado","soltero"),
                hijos=c("no","si","no","si","no"),
                n.hijos=c(NA,2,NA,1,NA),
                peso=c(25,6,5,7,8))
y
##   id carnet estad.civil hijos n.hijos peso
## 1  1     si     soltero    no      NA   25
## 2  2     si      casado    si       2    6
## 3  3     si     soltero    no      NA    5
## 4  4     no  divorciado    si       1    7
## 5  5     no     soltero    no      NA    8
Al aparece una misma variable en los dos data frame, al unirles crea dos columnas una peso.x y la otra peso.y indicando de esta forma de donde procede cada una de ellas.

merge(x,y, by="id")
##   id  nombre edad peso.x carnet estad.civil hijos n.hijos peso.y
## 1  1    Jos   30     75     si     soltero    no      NA     25
## 2  2   Pilar   40     60     si      casado    si       2      6
## 3  3   Nuria   22     55     si     soltero    no      NA      5
## 4  4    Pepa   53     78     no  divorciado    si       1      7
## 5  5 Antonio   27     80     no     soltero    no      NA      8
6.1.4 Dos data frame sin una variable clave comn como campo clave
Supongamos ahora que eliminamos del data frame la segunda fila. Es decir, los dos data frame tienen una columna que se llama id pero que no coincide en el contenido.

y <- y[-2,]
y
##   id carnet estad.civil hijos n.hijos peso
## 1  1     si     soltero    no      NA   25
## 3  3     si     soltero    no      NA    5
## 4  4     no  divorciado    si       1    7
## 5  5     no     soltero    no      NA    8
Y volvemos a realizar la unin como antes.

merge(x,y,by="id")
##   id  nombre edad peso.x carnet estad.civil hijos n.hijos peso.y
## 1  1    Jos   30     75     si     soltero    no      NA     25
## 2  3   Nuria   22     55     si     soltero    no      NA      5
## 3  4    Pepa   53     78     no  divorciado    si       1      7
## 4  5 Antonio   27     80     no     soltero    no      NA      8
En la tabla de datos final no aparece el individuo 2 (de nombre "Pilar"). Por defecto, la funcin merge() solo utiliza las filas que estn en los dos data frame.

Este comportamiento se puede dirigir por medio de los argumentos: all,all.x y all.y

Si se especifica:
  
  all=TRUE se utilizan todas las filas tanto del data frame x como del y.
all.x=TRUE se utilizan todas las filas del data frame x.
all.y=TRUE se utilizan todas las filas del data frame y.
En nuestro ejemplo son equivalentes los dos cdigos siguientes:
  
  merge(x,y, all=TRUE)
##   id peso  nombre edad carnet estad.civil hijos n.hijos
## 1  1   25    <NA>   NA     si     soltero    no      NA
## 2  1   75    Jos   30   <NA>        <NA>  <NA>      NA
## 3  2   60   Pilar   40   <NA>        <NA>  <NA>      NA
## 4  3    5    <NA>   NA     si     soltero    no      NA
## 5  3   55   Nuria   22   <NA>        <NA>  <NA>      NA
## 6  4    7    <NA>   NA     no  divorciado    si       1
## 7  4   78    Pepa   53   <NA>        <NA>  <NA>      NA
## 8  5    8    <NA>   NA     no     soltero    no      NA
## 9  5   80 Antonio   27   <NA>        <NA>  <NA>      NA
merge(x,y, all.x=TRUE)
##   id peso  nombre edad carnet estad.civil hijos n.hijos
## 1  1   75    Jos   30   <NA>        <NA>  <NA>      NA
## 2  2   60   Pilar   40   <NA>        <NA>  <NA>      NA
## 3  3   55   Nuria   22   <NA>        <NA>  <NA>      NA
## 4  4   78    Pepa   53   <NA>        <NA>  <NA>      NA
## 5  5   80 Antonio   27   <NA>        <NA>  <NA>      NA
Como se puede apreciar en el ejemplo, cuando se utilizan todas las filas los valores faltantes se rellenan con NA.

6.2 Combinar ms de dos data frame. La funcin Reduce()
La funcin merge permite unir dos tablas de datos en el caso de querer unir ms de dos se puede repetir el procediemento las veces que sea necesario.

Ejemplo 12

Se quieren unir los siguientes data frame,

A <- data.frame(id = c("A", "B", "C", "D"), edad = c(24, 25, 17, 19), altura = c(180,190, 175, 165))
A
##   id edad altura
## 1  A   24    180
## 2  B   25    190
## 3  C   17    175
## 4  D   19    165
B <- data.frame(gender = c("H", "H", "M", "M"), id = c("A", "B", "C", "D"))
B
##   gender id
## 1      H  A
## 2      H  B
## 3      M  C
## 4      M  D
C <- data.frame(id = c("A", "B", "C", "D"),
                mates = c(6.5, 8.9, 7.4, 9.2), 
                ciencias = c(7.2,8.4, 6.5, 8.7))
C
##   id mates ciencias
## 1  A   6.5      7.2
## 2  B   8.9      8.4
## 3  C   7.4      6.5
## 4  D   9.2      8.7
La primera opcin es aplicar sucesivamente la funcin merge()

AB <- merge(A,B)
AB
##   id edad altura gender
## 1  A   24    180      H
## 2  B   25    190      H
## 3  C   17    175      M
## 4  D   19    165      M
ABC <- merge (AB,C)
ABC
##   id edad altura gender mates ciencias
## 1  A   24    180      H   6.5      7.2
## 2  B   25    190      H   8.9      8.4
## 3  C   17    175      M   7.4      6.5
## 4  D   19    165      M   9.2      8.7
La otra opcin es utilizar la funcin Reduce()

Reduce(merge,list(A,B,C))
##   id edad altura gender mates ciencias
## 1  A   24    180      H   6.5      7.2
## 2  B   25    190      H   8.9      8.4
## 3  C   17    175      M   7.4      6.5
## 4  D   19    165      M   9.2      8.7
La funcin Reduce() permite aplicar una funcin binaria, en este caso merge(), sucesivamente a los elementos de una lista.

7 Eliminar filas duplicadas en un data frame
****Ejemplo 13***
  
  a <- c(rep("A", 3), rep("B", 3), rep("C",2))
b <- c(1,1,2,4,1,1,2,2)
df <-data.frame(a,b)
df
##   a b
## 1 A 1
## 2 A 1
## 3 A 2
## 4 B 4
## 5 B 1
## 6 B 1
## 7 C 2
## 8 C 2
Dos posibilidades,

duplicated(df)
## [1] FALSE  TRUE FALSE FALSE FALSE  TRUE FALSE  TRUE
df[duplicated(df), ]
##   a b
## 2 A 1
## 6 B 1
## 8 C 2
df[!duplicated(df), ]
##   a b
## 1 A 1
## 3 A 2
## 4 B 4
## 5 B 1
## 7 C 2
O utilizar la funcin

unique(df)
##   a b
## 1 A 1
## 3 A 2
## 4 B 4
## 5 B 1
## 7 C 2
8 Dividir un data frame. La funcin split()
La funcin subset()permite seleccionar un subconjunto de individuos de un data frame determinado. Cuando se quiere dividir un data frame en funcin de los valores de una determinada columna (variable) es ms eficiente utilizar la funcin split().

La funcin split() de sintaxis,

split(x, f, drop = FALSE, ...)

split(x, f, drop = FALSE, ...) <- value

unsplit(value, f, drop = FALSE)
donde

Nombre	Descripcin
x	vector o data frame que contiene los datos
f	factor en el sentido de variable que define los grupos. Permite iteracciones.
drop	variable lgica. Si toma el valor TRUEno utiliza los valores de fque no esten presentes en x.
value	lista de vectores o data frames com platibles con la dividin de xen funcin de f.
.	argumentos opcionales pasados por los methods.
Ejemplo 14

Del data frame iris se quiere dividirlo en funcin de la variable Specie

#Data frame iris
head(iris)
##   Sepal.Length Sepal.Width Petal.Length Petal.Width Species
## 1          5.1         3.5          1.4         0.2  setosa
## 2          4.9         3.0          1.4         0.2  setosa
## 3          4.7         3.2          1.3         0.2  setosa
## 4          4.6         3.1          1.5         0.2  setosa
## 5          5.0         3.6          1.4         0.2  setosa
## 6          5.4         3.9          1.7         0.4  setosa
dividido <- split(x = iris, f = iris$Species)
str(dividido)
## List of 3
##  $ setosa    :'data.frame':  50 obs. of  5 variables:
##   ..$ Sepal.Length: num [1:50] 5.1 4.9 4.7 4.6 5 5.4 4.6 5 4.4 4.9 ...
##   ..$ Sepal.Width : num [1:50] 3.5 3 3.2 3.1 3.6 3.9 3.4 3.4 2.9 3.1 ...
##   ..$ Petal.Length: num [1:50] 1.4 1.4 1.3 1.5 1.4 1.7 1.4 1.5 1.4 1.5 ...
##   ..$ Petal.Width : num [1:50] 0.2 0.2 0.2 0.2 0.2 0.4 0.3 0.2 0.2 0.1 ...
##   ..$ Species     : Factor w/ 3 levels "setosa","versicolor",..: 1 1 1 1 1 1 1 1 1 1 ...
##  $ versicolor:'data.frame':  50 obs. of  5 variables:
##   ..$ Sepal.Length: num [1:50] 7 6.4 6.9 5.5 6.5 5.7 6.3 4.9 6.6 5.2 ...
##   ..$ Sepal.Width : num [1:50] 3.2 3.2 3.1 2.3 2.8 2.8 3.3 2.4 2.9 2.7 ...
##   ..$ Petal.Length: num [1:50] 4.7 4.5 4.9 4 4.6 4.5 4.7 3.3 4.6 3.9 ...
##   ..$ Petal.Width : num [1:50] 1.4 1.5 1.5 1.3 1.5 1.3 1.6 1 1.3 1.4 ...
##   ..$ Species     : Factor w/ 3 levels "setosa","versicolor",..: 2 2 2 2 2 2 2 2 2 2 ...
##  $ virginica :'data.frame':  50 obs. of  5 variables:
##   ..$ Sepal.Length: num [1:50] 6.3 5.8 7.1 6.3 6.5 7.6 4.9 7.3 6.7 7.2 ...
##   ..$ Sepal.Width : num [1:50] 3.3 2.7 3 2.9 3 3 2.5 2.9 2.5 3.6 ...
##   ..$ Petal.Length: num [1:50] 6 5.1 5.9 5.6 5.8 6.6 4.5 6.3 5.8 6.1 ...
##   ..$ Petal.Width : num [1:50] 2.5 1.9 2.1 1.8 2.2 2.1 1.7 1.8 1.8 2.5 ...
##   ..$ Species     : Factor w/ 3 levels "setosa","versicolor",..: 3 3 3 3 3 3 3 3 3 3 ...
length(dividido)
## [1] 3
lista.Setosa <- dividido[1]
str(lista.Setosa)
## List of 1
##  $ setosa:'data.frame':  50 obs. of  5 variables:
##   ..$ Sepal.Length: num [1:50] 5.1 4.9 4.7 4.6 5 5.4 4.6 5 4.4 4.9 ...
##   ..$ Sepal.Width : num [1:50] 3.5 3 3.2 3.1 3.6 3.9 3.4 3.4 2.9 3.1 ...
##   ..$ Petal.Length: num [1:50] 1.4 1.4 1.3 1.5 1.4 1.7 1.4 1.5 1.4 1.5 ...
##   ..$ Petal.Width : num [1:50] 0.2 0.2 0.2 0.2 0.2 0.4 0.3 0.2 0.2 0.1 ...
##   ..$ Species     : Factor w/ 3 levels "setosa","versicolor",..: 1 1 1 1 1 1 1 1 1 1 ...
cuadro.Setosa <- dividido[[1]]
str(cuadro.Setosa)
## 'data.frame':    50 obs. of  5 variables:
##  $ Sepal.Length: num  5.1 4.9 4.7 4.6 5 5.4 4.6 5 4.4 4.9 ...
##  $ Sepal.Width : num  3.5 3 3.2 3.1 3.6 3.9 3.4 3.4 2.9 3.1 ...
##  $ Petal.Length: num  1.4 1.4 1.3 1.5 1.4 1.7 1.4 1.5 1.4 1.5 ...
##  $ Petal.Width : num  0.2 0.2 0.2 0.2 0.2 0.4 0.3 0.2 0.2 0.1 ...
##  $ Species     : Factor w/ 3 levels "setosa","versicolor",..: 1 1 1 1 1 1 1 1 1 1 ...
9 Familia apply en data frame
La familia de funciones *apply permiten sustituir los bucles clsicos. El uso de este tipo de funciones crea unos cdigos ms sencillos y claros de interpretar aunque no aumentan significativamente la velocidad de ejecucin.

Dentro de la familia *apply encontramos las funciones:
  
  apply() que trabaja con objetos matrix.
lapply() que trabaja con objetos list
tapply() que trabaja con objetos vector (y otros)
Todas estas funciones son aplicables a los data frame.

9.1 La funcin lapply()
La funcin lapply() con sintaxis,

lapply(X, FUN, .)

donde

Nombre	Descripcin
X	vector, lista o un objeto expression
FUN	funcin que se aplica a cada uno de los elementos de la lista
.	argumentos opcionales
aplica la funcin FUN acada uno de los elementos de la lista. Esta funcin siempre devuelve un objeto lista (list)

La funcin lapply()tiene un conjunto de funciones derivadas de ella:
  
  sapply() es similar a lappy() pero intenta devolver un objeto ms sencillo que una lista, es decir un objeto vector o matrix.

vapply() es similar a sapply()pero siempre simplifica la salida.

mapply() es una versin multivariante de sapply()'. La funcinmapply()aplicaFUN` al primer elemento de cada argumento, despus al segundo elemento de cada argumento, y as sucesivamente.

9.2 La funcin by()
La funcin by() no tiene la sintaxis tpica de una funcin *apply(). Sin embargo, es una funcin que realiza un trabajo semejante a la funcin  tapply() sobre los data frame.

La sintaxis de la funcin by() es,

by(data, INDICES, FUN, ., simplify = TRUE)

Nombre	Descripcin
data	objeto de R, data frame o matrix
INDICES	factor o lista de factores de igual longitud que data
FUN	funcin que se aplica a cada uno de los subconjuntos de data
.	argumentos opcionales de FUN.
simplify	varible lgica. Si toma el valor TRUE devuelve un array de tipo `list
Ejemplo 15

Dado el data frame iris. Se pide:

calcula la media de la longitud del sepalo en funcin de la variable Species
calcula la matriz de correlacin entre las dos primeras variables de iris
by(data = iris[, 1], INDICES = iris$Species, summary)
## iris$Species: setosa
##    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
##   4.300   4.800   5.000   5.006   5.200   5.800 
## -------------------------------------------------------- 
## iris$Species: versicolor
##    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
##   4.900   5.600   5.900   5.936   6.300   7.000 
## -------------------------------------------------------- 
## iris$Species: virginica
##    Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
##   4.900   6.225   6.500   6.588   6.900   7.900
by(data = iris[, 1:2], INDICES = iris$Species, cor)
## iris$Species: setosa
##              Sepal.Length Sepal.Width
## Sepal.Length    1.0000000   0.7425467
## Sepal.Width     0.7425467   1.0000000
## -------------------------------------------------------- 
## iris$Species: versicolor
##              Sepal.Length Sepal.Width
## Sepal.Length    1.0000000   0.5259107
## Sepal.Width     0.5259107   1.0000000
## -------------------------------------------------------- 
## iris$Species: virginica
##              Sepal.Length Sepal.Width
## Sepal.Length    1.0000000   0.4572278
## Sepal.Width     0.4572278   1.0000000
9.3 La funcin tapply()
La funcin tapply() con sintaxis,

tapply(X, INDEX, FUN = NULL, ., simplify = TRUE)

Nombre	Descripcin
X	vector
INDEX	lista de uno o ms factores con la misma longitud de X
FUN	funcin que se aplica a cada uno de los elementos de la lista. Entre otras permite la funcin NULL
.	argumentos opcionales de FUN.
simplify	varible lgica. Si toma el valor TRUE devuelve un array de tipo `list
Ejemplo 16 Dado el vector x=(1,2,3,4,5) y el factor fact=(A,A,B,B,B) se quiere encontrar la suma de cada conjunto definido por el factor.

x <- 1:5
fact <- c(rep("A",2), rep("B", 3))

tapply(x, fact, sum)
##  A  B 
##  3 12
10 Lectura de datos
La funcin read.table() es la forma de leer datos planos en R. Lee los datos de un fichero externo ASCII y los almacena en R como un objeto de tipo data frame.

Sus principales variantes son:

Nombre	Accin
read.csv()	Es read.table() con la apcin sep=","
read.fwf()	Es read.table() para datos con formato fijo
La sintaxis de read.table() es

read.table(file, header = FALSE, sep = "", quote = "\"'",
dec = ".", numerals = c("allow.loss", "warn.loss", "no.loss"),
row.names, col.names, as.is = !stringsAsFactors,
na.strings = "NA", colClasses = NA, nrows = -1,
skip = 0, check.names = TRUE, fill = !blank.lines.skip,
strip.white = FALSE, blank.lines.skip = TRUE,
comment.char = "#",
allowEscapes = FALSE, flush = FALSE,
stringsAsFactors = default.stringsAsFactors(),
fileEncoding = "", encoding = "unknown", text, skipNul = FALSE)
read.csv(file, header = TRUE, sep = ",", quote = "\"",
         dec = ".", fill = TRUE, comment.char = "", ...)

read.csv2(file, header = TRUE, sep = ";", quote = "\"",
          dec = ",", fill = TRUE, comment.char = "", ...)

read.delim(file, header = TRUE, sep = "\t", quote = "\"",
           dec = ".", fill = TRUE, comment.char = "", ...)

read.delim2(file, header = TRUE, sep = "\t", quote = "\"",
            dec = ",", fill = TRUE, comment.char = "", ...)
11 Atributos de un data frame
Un objeto data frame tiene los siguientes atributos

Atributo	Descripcin
length	nmero de columnas
mode	data frame
names	nombre de las columnas
------	----------